/* Title:	Globals.h 
* Author:	M.Thomas BEAM Ltd
* Date:		2007-06-01
*
* Contents:	
*
* Mod Rec:
*
*/


#ifndef GLOBALS_H
#define GLOBALS_H
#include <BString.h>
#include <Config.h>
#include <qstatusbar.h>

extern BString 		ghostName;
extern Config		gconfig;
extern QStatusBar*	gstatusbar;

#endif
