//******************************************************************************
//	Base objects for BEAM database development system
//******************************************************************************
//
module Base {

class BObjData {
	Id		id;
	String		name;
	String		domain;
	Date		dateCreated;
	Date		dateAccessed;
	Date		dateModified;
	List<String>	keywords;
};

class BObjDInfo : BObjData {
	String		description;
};

class BObjDTodo : BObjData {
	Int32		priority;
	Int32		done;
	Date		startDate;
	Date		endDate;
	String		description;
};

class BObjDAddress : BObjData {
	String		firstName;
	String		lastName;
	String		company;
	String		address;
	String		postCode;
	String		tel;
	String		mobile;
	String		fax;
	String		email;
	String		web;
	String		groups;
	String		christmas;
	String		misc;
	String		telWork;
};

class BObjDTest : BObjData {
	String		test1;
	String		test2;
};

class BObjDWhatsOn : BObjData {
	Date		date;
	Date		dateLast;
	String		tcGroup;
	String		startTime;
	String		startPlace;
	String		rideLength;
	String		ease;
	String		description;
	String		who;
	String		tel;
	String		email;
	String		web;
};

class BObjDMttest : BObjData {
	String		field1;
	String		field2;
};

class Item : BObjData {
	Int32		qty;
	String		info;
};

class Despatch : BObjData {
	Int32		start;
	Item		item1;
	Item		item2;
};

};
