/* Title:	Globals.cpp 
* Author:	M.Thomas BEAM Ltd
* Date:		2007-06-01
*
* Contents:	
*
* Mod Rec:
*
*/

#include <BString.h>
#include <Config.h>
#include <qstatusbar.h>

BString		ghostName;			// Global hostname from command line
Config		gconfig;			// Global access config file
QStatusBar*	gstatusbar;
