/*******************************************************************************
 *	BError.cc	BError Class
 *			T.Barnaby,	BEAM Ltd,	21/3/00
 *******************************************************************************
 */
#include <BError.h>


BError::BError(int errNo, BString errStr){
	set(errNo, errStr);
}

BError::BError(BString errStr){
	set(ERROR, errStr);
}

BError BError::copy(){
	return BError(oerrNo, oerrStr.copy());
}

BError& BError::set(int errNo, BString errStr){
	oerrNo = errNo;
	oerrStr = errStr;
	return *this;
}

BError& BError::setError(BString errStr){
	set(ERROR, errStr);
	return *this;
}

BString BError::getString() const {
	return oerrStr;
}

int BError::getErrorNo() const {
	return oerrNo;
}

BError::operator int () const {
	return oerrNo;
}
