/*******************************************************************************
 *	BoapnsC.h	Produced by Bidl
 *******************************************************************************
 */

#ifndef BOAPNSC_H
#define BOAPNSC_H 1

#include <stdlib.h>
#include <stdint.h>
#include <Boap.h>
#include <BString.h>
#include <BList.h>
#include <BArray.h>
#include <BoapnsD.h>


namespace Boapns {
	const BUInt32 apiVersion = 0;

	class Boapns : public BoapClientObject {
	public:
			Boapns(BString name = "");
		BError getVersion(BString& version);
		BError getEntryList(BList<BoapEntry>& entryList);
		BError getEntry(BString name, BoapEntry& entry);
		BError addEntry(BoapEntry entry);
		BError delEntry(BString name);
		BError getNewName(BString& name);
	private:
	};
}
#endif
